@@ -4,6 +4,7 @@ from typing import Optional, Dict, Any, List, Protocol
 from ...model.playbook_action import PlaybookAction
 from ...core.model.events import ExecutionBaseEvent
 from ...core.playbooks.base_trigger import TriggerEvent
+from ...runner.telemetry import Telemetry
 
 
 class PlaybooksEventHandler(Protocol):
@@ -19,6 +20,8 @@ class PlaybooksEventHandler(Protocol):
         self,
         execution_event: ExecutionBaseEvent,
         actions: List[PlaybookAction],
+        sync_response: bool = False,
+        no_sinks: bool = False,
     ) -> Optional[Dict[str, Any]]:
         """Run list of actions using the provided execution event"""
         pass
@@ -29,6 +32,8 @@ class PlaybooksEventHandler(Protocol):
         action_name: str,
         action_params: Optional[dict],
         sinks: Optional[List[str]],
+        sync_response: bool = False,
+        no_sinks: bool = False
     ) -> Optional[Dict[str, Any]]:
         """Execute an external action"""
         pass
@@ -39,3 +44,10 @@ class PlaybooksEventHandler(Protocol):
     ) -> dict:
         """Return runner global config"""
         pass
+
+    @abstractmethod
+    def get_telemetry(
+        self,
+    ) -> Telemetry:
+        """Return runner telemetry"""
+        pass
